@@ -3,7 +3,7 @@ package Promise::ES6;
 use strict;
 use warnings;
 
-our $VERSION = '0.20';
+our $VERSION = '0.20_01';
 
 =encoding utf-8
 
@@ -46,11 +46,6 @@ syntax.
 This is a rewrite of an earlier module, L<Promise::Tiny>. It fixes several
 bugs and superfluous dependencies in the original.
 
-=head1 STATUS
-
-This module is in use in production and, backed by a pretty extensive
-set of regression tests, may be considered stable.
-
 =head1 INTERFACE NOTES
 
 =over
@@ -61,7 +56,7 @@ not a list.
 =item * Unhandled rejections are reported via C<warn()>. (See below
 for details.)
 
-=item * The L<Promises/A+ test suite|https://github.com/promises-aplus/promises-tests> avoids testing the case where an “executor”
+=item * The Promises/A+ test suite avoids testing the case where an “executor”
 function’s resolve callback itself receives another promise, e.g.:
 
     my $p = Promise::ES6->new( sub ($res) {
@@ -87,7 +82,7 @@ Future is not compatible with promises.
 
 This module’s handling of unhandled rejections has changed over time.
 The current behavior is: if any rejected promise is DESTROYed without first
-having received a catch callback, a warning is thrown.
+having received a failure callback, a warning is thrown.
 
 =head1 SYNCHRONOUS OPERATION
 
@@ -123,7 +118,7 @@ alternatives.
 =head1 CANCELLATION
 
 Promises have never provided a standardized solution for cancellation—i.e.,
-aborting an in-process operation. If you need this functionality, then, you’ll
+aborting an in-process operation. So, if you need this functionality, you’ll
 have to implement it yourself. Two ways of doing this are:
 
 =over
@@ -152,18 +147,17 @@ Here are a few “pointers” (heh) to bear in mind:
 
 =over
 
-=item * Any Promise::ES6 instances that are created while
+=item * As of version 0.07, any Promise::ES6 instances that are created while
 C<$Promise::ES6::DETECT_MEMORY_LEAKS> is set to a truthy value are
 “leak-detect-enabled”, which means that if they survive until their original
-process’s global destruction, a warning is triggered. You should normally
-enable this flag in a development environment.
+process’s global destruction, a warning is triggered.
 
 =item * If your application needs recursive promises (e.g., to poll
 iteratively for completion of a task), the C<current_sub> feature (i.e.,
 C<__SUB__>) may help you avoid memory leaks. In Perl versions that don’t
-support this feature (i.e., anything pre-5.16) you can imitate it thus:
+support this feature you can imitate it thus:
 
-    use constant _has_current_sub => eval "use feature 'current_sub'";
+    use constant _has_current_sub => $^V ge v5.16.0;
 
     use if _has_current_sub(), feature => 'current_sub';
 
