@@ -25,6 +25,7 @@ use constant {
     _DETECT_LEAK_IDX => _DEBUG + 3,
     _ON_RESOLVE_IDX  => _DEBUG + 4,
     _ON_REJECT_IDX   => _DEBUG + 5,
+    _IS_FINALLY_IDX  => _DEBUG + 6,
 };
 
 # "$value_sr" => $value_sr
@@ -94,32 +95,21 @@ sub new {
     return $self;
 }
 
-sub finally {
-    my ($self, $on_finish) = @_;
-
-    my $new;
-    $new = ref($self)->new( sub {
-        my ($y, $n) = @_;
-
-        $self->then(
-            sub { $on_finish->(); $y->(@_) },
-            sub {
-                $on_finish->(); $n->(@_);
-
-                # This is here to delay $new’s GC until after its rejection.
-                # That delay facilitates the unhandled-rejection warning;
-                # if $new is GCed prior to its rejection then the
-                # unhandled-rejection warning won’t fire correctly.
-                undef $new;
-            },
-        );
-    } );
-
-    return $new;
+sub then {
+    return $_[0]->_then_or_finally(@_[1, 2]);
 }
 
-sub then {
-    my ( $self, $on_resolve, $on_reject ) = @_;
+sub finally {
+
+    # There’s no reason to call finally() without a callback
+    # since it would just be a no-op.
+    die 'finally() requires a callback!' if !$_[1];
+
+    return $_[0]->_then_or_finally($_[1], undef, 1);
+}
+
+sub _then_or_finally {
+    my ($self, $on_resolve_or_finish, $on_reject, $is_finally) = @_;
 
     my $value_sr = bless( \do { my $v }, _PENDING_CLASS() );
 
@@ -129,8 +119,9 @@ sub then {
         [],
         $value_sr,
         $Promise::ES6::DETECT_MEMORY_LEAKS,
-        $on_resolve,
+        $on_resolve_or_finish,
         $on_reject,
+        $is_finally,
       ],
       ref($self);
 
@@ -151,16 +142,23 @@ sub then {
 }
 
 sub _repromise {
-    my ( $value_sr, $children_ar, $repromise_value_sr ) = @_;
+    my ( $value_sr, $children_ar, $repromise_value_sr, $orig_finally_sr ) = @_;
     $$repromise_value_sr->then(
         sub {
-            $$value_sr = $_[0];
+            if (ref $orig_finally_sr) {
+                $$value_sr = $$orig_finally_sr;
+            }
+            else {
+                $$value_sr = $_[0];
+            }
+
             bless $value_sr, _RESOLUTION_CLASS;
             $_->_settle($value_sr) for splice @$children_ar;
         },
         sub {
             $$value_sr = $_[0];
             bless $value_sr, _REJECTION_CLASS;
+            $_UNHANDLED_REJECTIONS{$value_sr} = $$value_sr;
             $_->_settle($value_sr) for splice @$children_ar;
         },
     );
@@ -173,27 +171,29 @@ sub _repromise {
 #    return (_PENDING_CLASS ne ref $_[0][ _VALUE_SR_IDX ]);
 #}
 
-my $settle_is_rejection;
+my ($settle_is_rejection, $self_is_finally);
 
-# This method *only* runs when $self is “settled”:
+# This method *only* runs to “settle” a promise.
 sub _settle {
     my ( $self, $final_value_sr ) = @_;
 
     die "$self already settled!" if _PENDING_CLASS ne ref $self->[_VALUE_SR_IDX];
 
+    $self_is_finally = $self->[_IS_FINALLY_IDX];
+
     $settle_is_rejection = _REJECTION_CLASS eq ref $final_value_sr;
 
+    delete $_UNHANDLED_REJECTIONS{$final_value_sr} if $settle_is_rejection;
+
     # A promise that new() created won’t have on-settle callbacks,
     # but a promise that came from then/catch/finally will.
     # It’s a good idea to delete the callbacks in order to trigger garbage
     # collection as soon and as reliably as possible. It’s safe to do so
     # because _settle() is only called once.
-    my $callback = $self->[ $settle_is_rejection ? _ON_REJECT_IDX : _ON_RESOLVE_IDX ];
+    my $callback = $self->[ ($settle_is_rejection && !$self_is_finally) ? _ON_REJECT_IDX : _ON_RESOLVE_IDX ];
 
     @{$self}[ _ON_RESOLVE_IDX, _ON_REJECT_IDX ] = ();
 
-    delete $_UNHANDLED_REJECTIONS{$final_value_sr} if $settle_is_rejection;
-
     # In some contexts this function runs quite a lot,
     # so caching the is-promise lookup is useful.
     my $value_sr_contents_is_promise = 1;
@@ -208,7 +208,7 @@ sub _settle {
 
         local $@;
 
-        if ( eval { $new_value = $callback->($$final_value_sr); 1 } ) {
+        if ( eval { $new_value = $callback->($self_is_finally ? () : $$final_value_sr); 1 } ) {
 
             # The callback succeeded. If $new_value is not itself a promise,
             # then $self is now resolved. (Yay!) Note that this is true
@@ -218,7 +218,23 @@ sub _settle {
             # If $new_value IS a promise, though, then we have to wait.
             if ( !UNIVERSAL::can( $new_value, 'then' ) ) {
                 $value_sr_contents_is_promise = 0;
-                bless $self->[_VALUE_SR_IDX], _RESOLUTION_CLASS();
+
+                if ($self_is_finally) {
+
+                    # finally() is a bit weird. Assuming its callback succeeds,
+                    # it takes its parent’s resolution state. It’s important
+                    # that we make a *new* reference to the resolution value,
+                    # though, rather than merely using $final_value_sr itself,
+                    # because we need $self to have its own entry in
+                    # %_UNHANDLED_REJECTIONS.
+                    ${ $self->[_VALUE_SR_IDX] } = $$final_value_sr;
+                    bless $self->[_VALUE_SR_IDX], ref $final_value_sr;
+
+                    $_UNHANDLED_REJECTIONS{ $self->[_VALUE_SR_IDX] } = $self->[_VALUE_SR_IDX] if $settle_is_rejection;
+                }
+                else {
+                    bless $self->[_VALUE_SR_IDX], _RESOLUTION_CLASS;
+                }
             }
         }
         else {
@@ -232,7 +248,9 @@ sub _settle {
             $_UNHANDLED_REJECTIONS{ $self->[_VALUE_SR_IDX] } = $self->[_VALUE_SR_IDX];
         }
 
-        ${ $self->[_VALUE_SR_IDX] } = $new_value;
+        if (!$self_is_finally || $value_sr_contents_is_promise) {
+            ${ $self->[_VALUE_SR_IDX] } = $new_value;
+        }
     }
     else {
 
@@ -240,6 +258,8 @@ sub _settle {
         # indicates # (i.e., resolution or rejection) is now $self’s state
         # as well.
 
+        # NB: We should NEVER be here if the promise is from finally().
+
         bless $self->[_VALUE_SR_IDX], ref($final_value_sr);
         ${ $self->[_VALUE_SR_IDX] } = $$final_value_sr;
         $value_sr_contents_is_promise = UNIVERSAL::can( $$final_value_sr, 'then' );
@@ -250,7 +270,15 @@ sub _settle {
     }
 
     if ($value_sr_contents_is_promise) {
-        return _repromise( @{$self}[ _VALUE_SR_IDX, _CHILDREN_IDX, _VALUE_SR_IDX ] );
+
+        # Stash the given concrete value. If the $value_sr promise
+        # rejects, then we’ll accept that, but if it resolves, then
+        # we’ll look at this to know to discard that resolution.
+        if ($self_is_finally) {
+            $self->[_IS_FINALLY_IDX] = $final_value_sr;
+        }
+
+        return _repromise( @{$self}[ _VALUE_SR_IDX, _CHILDREN_IDX, _VALUE_SR_IDX, _IS_FINALLY_IDX ] );
     }
 
     if ( @{ $self->[_CHILDREN_IDX] } ) {
